#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>

#define BAUDRATE B19200
#define DEVICE "/dev/ttyUSB0"

#define DELAY_LOOP 10 //In sec

//COMUNCAZIONE CON SX16
#define START 0x33
#define RETURN_START 0x22
#define ADDR 0x01
#define WRITE_CMD 0x03 
#define READ_CMD 0x01 
#define ALL_CMD 0xFF
#define CH 0x01
#define ON 0x01
#define OFF 0x00
#define ZERO_CHAR 0x00

 
///////////////////////
//PROTOTIPI
///////////////////////
int read_key();
int send2sx16 (int _fd, char *cmd2sx16, char *sx16return);

///////////////////////
//FUNZIONI
///////////////////////
//Legge l'input da tastiera
int read_key(){
    int cmd;
    while (cmd = getchar()){ 
      if (cmd!=10) break;
    }
    return cmd;
}
//Invia un comando alla SX16
int send2sx16 (int _fd, char *cmd2sx16, char *sx16return){
  int res;
  
  if (write(_fd, cmd2sx16, 6) < 0) {
    //Invio sulla seriale fallito
    return 0;
  } else {
    //Invio del comando riuscito
    usleep(100000); //sleep per 100mSec
    res = read(_fd,sx16return,6);
    sx16return[res]=0;
    return 1;
  }
}


////////////////////////////////
//
// MAIN
//
////////////////////////////////
main (int argc, char * argv[]) {
  int fd, res, i;
  int b;
  int cmd;
  struct termios oldtio,newtio;
  char buf[10];
  
  system("clear"); //Pulizia della console
  printf("Apertua Porta COM\n");
  fd = open(DEVICE, O_RDWR | O_NOCTTY);
  if (fd < 0 ) {
    printf("Device %s non pesente su questo sistema\n\n", DEVICE);
    exit(-1);
  }
  printf("Inizializzazione della comunicazione seriale\n");
  
  tcgetattr(fd,&oldtio); /* Salva i precedenti settaggi */
  
  bzero(&newtio, sizeof(newtio));
  newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
  newtio.c_iflag = IGNPAR;
  newtio.c_oflag = 0;
  newtio.c_lflag = 0;
  newtio.c_cc[VTIME]    = 0;
  newtio.c_cc[VMIN]     = 6; 
  
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);
  //Inizializzazione device seriale terminata
  while (1){
    system("clear");
    //FORMATTO IL PACCHETTO DA MANDARE ALLA SERIALE
    buf[0]=0;
    buf[0]=START;
    buf[1]=ADDR;
    buf[2]=READ_CMD;
    buf[3]=ZERO_CHAR;
    buf[4]=ZERO_CHAR;
    buf[5]=ZERO_CHAR;
    if (send2sx16 (fd, buf, buf)) {
      if((buf[0]==RETURN_START) && (buf[1]==ADDR) && (buf[2]==READ_CMD)) {
        printf("---------------------------------------\n");
        printf(" LINUX 2 SX16\n\n");
        printf(" Ver: 1.0\n");
        printf(" Author: Daniele De Santis\n");
        printf(" E-Mail: desantis@areasx.it\n\n");
        printf(" STATO DELLE LINEE D'INPUT SULLA SX 16 \n");
        printf("---------------------------------------\n");

        printf("Stato degli ingressi CLC:\n");
        for (i=0; i<=7; i++){
          b = buf[3] & 1;
          buf[3] >>=1;
          if(b)
            printf("LINEA CLC %d    [X]  ON\n", (i + 1) );
          else
            printf("LINEA CLC %d    [ ]  OFF\n", (i + 1));
        }
        printf("Stato degli ingressi TTL:\n");
        for (i=0; i<=7; i++){
          b = buf[4] & 1;
          buf[4] >>=1;
          if(b)
            printf("LINEA TTL %d    [X]  ON\n", (i + 1) );
          else
            printf("LINEA TTL %d    [ ]  OFF\n", (i + 1));
        }
        printf("Stato degli ingressi OPTOISOLATE:\n");
        for (i=0; i<=7; i++){
          b = buf[5] & 1;
          buf[5] >>=1;
          if(b)
            printf("LINEA OPTO %d   [X]  ON\n", (i + 1) );
          else
            printf("LINEA OPTO %d   [ ]  OFF\n", (i + 1));
        }
        
      }  else {
        printf("ERRORE -Risposta dalla SX16 non valida\n");
      }
      printf("\nCTRL+C per uscire\n");
      usleep(1000000 * DELAY_LOOP);
    } else {
      printf("ERRORE -Comunicazione con il device %s fallito\n", DEVICE);
      exit(-1);
    }
  }
  printf("\nChiusura comunicazione seriale...\n");
  tcsetattr(fd,TCSANOW,&oldtio);
  printf("Arrivederci :)\n");
  close(fd);
  exit(0);
}
