#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>

#define BAUDRATE B19200
#define DEVICE "/dev/ttyUSB0"


//COMUNCAZIONE CON SX16
#define START 0x33
#define RETURN_START 0x22
#define ADDR 0x01
#define WRITE_CMD 0x03 
#define ALL_CMD 0xFF
#define CH 0x01
#define ON 0x01
#define OFF 0x00
#define ZERO_CHAR 0x00

 
///////////////////////
//PROTOTIPI
///////////////////////
int read_key();
int send2sx16 (int _fd, char *cmd2sx16, char *sx16return);

///////////////////////
//FUNZIONI
///////////////////////
//Legge l'input da tastiera
int read_key(){
    int cmd;
    while (cmd = getchar()){ 
      if (cmd!=10) break;
    }
    return cmd;
}
//Invia un comando alla SX16
int send2sx16 (int _fd, char *cmd2sx16, char *sx16return){
  int res;
  
  if (write(_fd, cmd2sx16, 6) < 0) {
    //Invio sulla seriale fallito
    return 0;
  } else {
    //Invio del comando riuscito
    usleep(100000); //sleep per 100mSec
    res = read(_fd,sx16return,6);
    sx16return[res]=0;
    return 1;
  }
}


////////////////////////////////
//
// MAIN
//
////////////////////////////////
main (int argc, char * argv[]) {
  int fd, res, i;
  int b;
  int cmd;
  int line_rele;
  int state_rele;
  struct termios oldtio,newtio;
  char buf[10];
  
  system("clear"); //Pulizia della console
  printf("----------------------------\n");
  printf(" LINUX 2 SX16\n\n");
  printf(" Ver: 1.0\n");
  printf(" Author: Daniele De Santis\n");
  printf(" E-Mail: desantis@areasx.it\n\n");
  printf(" Test RELE'\n");
  printf("----------------------------\n");
  
  printf("Apertua Porta COM\n");
  fd = open(DEVICE, O_RDWR | O_NOCTTY);
  if (fd < 0 ) {
    printf("Device %s non pesente su questo sistema\n\n", DEVICE);
    exit(-1);
  }
  printf("Inizializzazione della comunicazione seriale\n");
  
  tcgetattr(fd,&oldtio); /* Salva i precedenti settaggi */
  
  bzero(&newtio, sizeof(newtio));
  newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
  newtio.c_iflag = IGNPAR;
  newtio.c_oflag = 0;
  newtio.c_lflag = 0;
  newtio.c_cc[VTIME]    = 0;
  newtio.c_cc[VMIN]     = 6; 
  
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);
  //Inizializzazione device seriale terminata
  
  printf("Seleziona il rel da settare [1 - 6]\n");
  line_rele = read_key();
  if(line_rele < 49 && line_rele > 54) line_rele=49;
  
  printf("Seleziona lo stato [1 ON - 0 OFF]\n");
  state_rele = read_key();
  if (state_rele!=48 || state_rele!=49)  state_rele == 48;
  //printf("RELE' %d STATO %d\n", (line_rele - 48), (state_rele - 48));
            
  //FORMATTO IL PACCHETTO DA MANDARE ALLA SERIALE
  buf[0]=0;
  buf[0]=START;
  buf[1]=ADDR;
  buf[2]=WRITE_CMD;
  buf[3]=(line_rele - 48);
  if(state_rele==49)
    buf[4]=ON;
  else
    buf[4]=OFF;
  buf[5]=ZERO_CHAR;
  if (send2sx16 (fd, buf, buf)) {
    if((buf[0]==RETURN_START) && (buf[1]==ADDR) && (buf[2]==WRITE_CMD)) {
      printf("----------------------------\n");
      printf(" STATO DEI RELE SULLA SX 16 \n");
      printf("----------------------------\n");
      res = buf[3];
      for (i=0; i<=5; i++){
        b = res & 1;
        res >>=1;
        
        if(b){
          printf("RELE' %d   [X]  ON\n", (i + 1) );
        } else {
          printf("RELE' %d   [ ]  OFF\n", (i + 1));
        }
      }
    }
  }
  printf("\nChiusura comunicazione seriale...\n");
  tcsetattr(fd,TCSANOW,&oldtio);
  printf("Arrivederci :)\n");
  close(fd);
  exit(0);
}
