#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>

#define BAUDRATE B19200
#define DEVICE "/dev/ttyUSB0"

#define DELAY_LOOP 10 //In sec

//COMUNCAZIONE CON SX16
#define START 0x33
#define RETURN_START 0x22
#define ADDR 0x01
#define WRITE_CMD 0x03 
#define TMP_CMD 0x02 
#define READ_CMD 0x01 
#define ALL_CMD 0xFF
#define CH 0x01
#define ON 0x01
#define OFF 0x00
#define ZERO_CHAR 0x00
#define TEST 0x80

///////////////////////
//PROTOTIPI
///////////////////////
int read_key();
int send2sx16 (int _fd, char *cmd2sx16, char *sx16return);

///////////////////////
//FUNZIONI
///////////////////////
//Legge l'input da tastiera
int read_key(){
    int cmd;
    while (cmd = getchar()){ 
      if (cmd!=10) break;
    }
    return cmd;
}
//Invia un comando alla SX16
int send2sx16 (int _fd, char *cmd2sx16, char *sx16return){
  int res;
  char buffer[7];
  if (write(_fd, cmd2sx16, 6) < 0) {
    //Invio sulla seriale fallito
    return 0;
  } else {
    //Invio del comando riuscito
    usleep(500000); //sleep per 500mSec
    res = read(_fd,sx16return,6);
    return 1;
  }
}

////////////////////////////////
//
// MAIN
//
////////////////////////////////
main (int argc, char * argv[]) {
  int fd, res, i;
  auto float temp;
  int cmd;
  struct termios oldtio,newtio;
  unsigned char buf[20];
  
  system("clear"); //Pulizia della console
  printf("Apertua Porta COM\n");
  fd = open(DEVICE, O_RDWR | O_NOCTTY);
  if (fd < 0 ) {
    printf("Device %s non pesente su questo sistema\n\n", DEVICE);
    exit(-1);
  }
  printf("Inizializzazione della comunicazione seriale\n");
  tcgetattr(fd,&oldtio); /* Salva i precedenti settaggi */
  
  bzero(&newtio, sizeof(newtio));
  newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
  newtio.c_iflag = IGNPAR;
  newtio.c_oflag = 0;
  newtio.c_lflag = 0;
  newtio.c_cc[VTIME]    = 0;
  newtio.c_cc[VMIN]     = 5; 
  
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);
  //Inizializzazione device seriale terminata
  while (1){
    system("clear");
    //FORMATTO IL PACCHETTO DA MANDARE ALLA SERIALE
    buf[0]=0;
    buf[0]=START;
    buf[1]=ADDR;
    buf[2]=TMP_CMD;
    buf[3]=ZERO_CHAR;
    buf[4]=ZERO_CHAR;
    buf[5]=ZERO_CHAR;
    if (send2sx16 (fd, buf, buf)) {
      if((buf[0]==RETURN_START) && (buf[1]==ADDR) && (buf[2]==TMP_CMD)) {
        printf("---------------------------------------\n");
        printf(" LINUX 2 SX16\n\n");
        printf(" Ver: 1.0\n");
        printf(" Author: Daniele De Santis\n");
        printf(" E-Mail: desantis@areasx.it\n\n");
        printf(" LETTURA DELLA TEMPERATURA DALLA SX 16 \n");
        printf("---------------------------------------\n");
        printf("Temperatura letta dal sensore DS1621: ");

        temp=0;
        // Imposto il mezzo grado
        if (buf[4]==0x80) temp=0.5;
        if (buf[3] & 0x80) {
          // Temperatura negativa
          temp+=(!buf[3])+1;
          temp=-temp;
        } else {
          // Temperatura >0
          temp+=buf[3];
        }
        printf("%.2f C\n", temp);
      }  else {
        printf("ERRORE -Risposta dalla SX16 non valida\n");
      }
      printf("\nCTRL+C per uscire\n");
      usleep(1000000 * DELAY_LOOP);
    } else {
      printf("ERRORE -Comunicazione con il device %s fallito\n", DEVICE);
      exit(-1);
    }
  }
  printf("\nChiusura comunicazione seriale...\n");
  tcsetattr(fd,TCSANOW,&oldtio);
  printf("Arrivederci :)\n");
  close(fd);
  exit(0);
}
